#!/bin/sh
# firewall.sh [p]
# Insert firewall rules for IPv4 and IPv6 allowing ping and traceroute as well as IPv4 NAT
# Also allowing some services
# This script may be run more than once without problems
# 2022-08-13 Joergen Thomsen bugs@jth.net

# Based on this hardware:
# Setup 1:
# Server plus router functions
# Ethernet socket 0: eth0 connected to ISP provider with DHCP IPv4 and IPv6. LAN address: eth0:1: 192.168.1.1/24
# Ethernet socket 1: eth1 Subnet: 192.168.2.0/24
# wlan0: 1 builtin wifi unit (as access point), low quality. Subnet: 10.0.1.0/24

# A switch is connected to eth1. To this switch a better access point and
# some PCs and a TV are connected
# A webcam is connected through the builtin wifi
# eth1 and/or wlan0 may not be present

# Setup 2:
# Server without router functions
# eth1 and wlan0 will not be present

# This script is checking for the presence of eth1 and wlan0
# (possibly renamed by /etc/udev/rules.d/70-persistent-net.rules)
# so it may also be used in a simple eth0 server setup

# Comment out or delete lines not needed,
#
# I keep this file in /etc/sysconfig/network-scripts/firewall.sh

# Please note /etc/sysconfig/firewalld
# FIREWALLD_ARGS=--debug=1

# See also Tips and tricks below

# ZONE may be set manually if not using the default zone
ZONE=`firewall-cmd --get-default-zone`

# --permanent will prevent the rules from disappearing at reboot
PERMANENT=${1:-}
if [ "$PERMANENT" = "p" ]; then
	PERMANENT=--permanent
else
if [ "$PERMANENT" != "" ]; then
PERMANENT=
echo $0 [p]
echo p = make rules permanent
exit
fi
fi

echo Updating zone $ZONE
ZONE=--zone=$ZONE

# start by disallowing all ICMPs
firewall-cmd $ZONE --add-icmp-block-inversion
# add ping rules
firewall-cmd $ZONE --add-icmp-block=echo-request
firewall-cmd $ZONE --add-icmp-block=echo-reply
#MTU discovery
firewall-cmd $ZONE --add-icmp-block=fragmentation-needed
# add ipv6 rules
firewall-cmd $ZONE --add-icmp-block=neighbour-solicitation
firewall-cmd $ZONE --add-icmp-block=neighbour-advertisement
firewall-cmd $ZONE --add-icmp-block=router-advertisement
firewall-cmd $ZONE --add-icmp-block=router-solicitation
firewall-cmd $ZONE --add-icmp-block=time-exceeded
firewall-cmd $ZONE --add-icmp-block=destination-unreachable
firewall-cmd $ZONE --add-icmp-block=address-unreachable
firewall-cmd $ZONE --add-icmp-block=port-unreachable
firewall-cmd $ZONE --add-icmp-block=network-unreachable
firewall-cmd $ZONE --add-icmp-block=redirect

#****** add services used by systemd-networkd
# display available standard services located in /usr/lib/firewalld/services
# firewall-cmd --get-services
# Create your own in /etc/firewalld/services

firewall-cmd $ZONE --add-service dhcp
firewall-cmd $ZONE --add-service dhcpv6
firewall-cmd $ZONE --add-service dhcpv6-client
firewall-cmd $ZONE --add-service dns

# for a basic web and mailserver
#firewall-cmd $ZONE --add-service http
#firewall-cmd $ZONE --add-service https
#firewall-cmd $ZONE --add-service smtp
#firewall-cmd $ZONE --add-service pop3


#****** IMPORTANT: IPv4 NAT function as router (masquerade)

# data out through the NAT engine
firewall-cmd $ZONE --add-interface eth0
echo firewall-cmd $ZONE --add-forward is not supported and will cause an error in older versions of firewalld
firewall-cmd $ZONE --add-forward


#****** eth0:1 local subnet of eth0 192.168.1.1/24
firewall-cmd $ZONE --add-interface eth0:1
firewall-cmd $ZONE --add-rich-rule='rule family=ipv4 source address=192.168.1.0/24 masquerade'

OTHERLINKS=0

if [ "`ip link show eth1 2>&1|head -c -1 -`" != 'Device "eth1" does not exist.' ]; then
OTHERLINKS=1
#****** eth1 subnet 192.168.2.0/24
firewall-cmd $ZONE --add-interface eth1
firewall-cmd $ZONE --add-rich-rule='rule family=ipv4 source address=192.168.2.0/24 masquerade'

# Handle access point at 192.168.2.2
# Login to it from outside using the public address at eth0 and port 4 http://<public address>:4
firewall-cmd $ZONE --add-forward-port=port=4:proto=tcp:toport=80:toaddr=192.168.2.2

fi

if [ "`ip link show wlan0 2>&1|head -c -1 -`" != 'Device "wlan0" does not exist.' ]; then
OTHERLINKS=2

#****** wlan0 subnet 10.0.1.0/24
firewall-cmd $ZONE --add-interface wlan0
firewall-cmd $ZONE --add-rich-rule='rule family=ipv4 source address=10.0.1.0/24 masquerade'

# Handle webcam at 10.0.1.2
# Login to it from outside using the public address at eth0 and port 2 or 3 http://<public address>:2 or3

firewall-cmd $ZONE --add-forward-port=port=2:proto=tcp:toport=80:toaddr=10.0.1.2
firewall-cmd $ZONE --add-forward-port=port=3:proto=tcp:toport=443:toaddr=10.0.1.2

fi

# OBS! If no eth1 and wlan0 this may be the only masquerade (NAT) command needed
[ $OTHERLINKS -eq 0 ] && firewall-cmd $ZONE --add-masquerade

if [ "$PERMANENT" = "--permanent" ]; then
  firewall-cmd --runtime-to-permanent
  firewall-cmd --check-config
  # Run the reload command manually after checking
  echo  if everything OK then
  echo  '  ' firewall-cmd --reload
else
# Run the reload command manually after checking
echo  if everything OK then make the configuration permanent by
echo  '  ' firewall-cmd --runtime-to-permanent
echo  '  ' firewall-cmd --reload
fi

#---- DONE ----
exit

# Tips and tricks

# Display the current configuration
firewall-cmd $ZONE --list-all

# Display all icmptypes
firewall-cmd --get-icmptypes

# Display all services
firewall-cmd --get-services

# Display info
ip -details link show [eth0]
ip a

